@@ -475,13 +475,28 @@ pub trait ExprExt {
             return (Pure, Known(false));
         }
 
-        let val = match *expr {
+        let val = match expr {
             Expr::Paren(ref e) => return e.expr.as_bool(),
             Expr::Assign(AssignExpr { ref right, .. }) => {
                 let (_, v) = right.as_bool();
                 return (MayBeImpure, v);
             }
 
+            Expr::Unary(UnaryExpr {
+                op: op!(unary, "-"),
+                arg,
+                ..
+            }) => {
+                let v = arg.as_number();
+                match v {
+                    Known(n) => Known(match n.classify() {
+                        FpCategory::Nan | FpCategory::Zero => false,
+                        _ => true,
+                    }),
+                    Unknown => return (MayBeImpure, Unknown),
+                }
+            }
+
             Expr::Unary(UnaryExpr {
                 op: op!("!"),
                 ref arg,
@@ -490,7 +505,58 @@ pub trait ExprExt {
                 let (p, v) = arg.as_bool();
                 return (p, !v);
             }
-            Expr::Seq(SeqExpr { ref exprs, .. }) => exprs.last().unwrap().as_bool().1,
+            Expr::Seq(SeqExpr { exprs, .. }) => exprs.last().unwrap().as_bool().1,
+
+            Expr::Bin(BinExpr {
+                left,
+                op: op!(bin, "-"),
+                right,
+                ..
+            }) => {
+                let (lp, ln) = left.cast_to_number();
+                let (rp, rn) = right.cast_to_number();
+
+                return (
+                    lp + rp,
+                    match (ln, rn) {
+                        (Known(ln), Known(rn)) => {
+                            if ln == rn {
+                                Known(false)
+                            } else {
+                                Known(true)
+                            }
+                        }
+                        _ => Unknown,
+                    },
+                );
+            }
+
+            Expr::Bin(BinExpr {
+                left,
+                op: op!("/"),
+                right,
+                ..
+            }) => {
+                let lv = left.as_number();
+                let rv = right.as_number();
+
+                match (lv, rv) {
+                    (Known(lv), Known(rv)) => {
+                        // NaN is false
+                        if lv == 0.0 && rv == 0.0 {
+                            return (Pure, Known(false));
+                        }
+                        // Infinity is true.
+                        if rv == 0.0 {
+                            return (Pure, Known(true));
+                        }
+                        let v = lv / rv;
+
+                        return (Pure, Known(v != 0.0));
+                    }
+                    _ => Unknown,
+                }
+            }
 
             Expr::Bin(BinExpr {
                 ref left,
@@ -509,7 +575,7 @@ pub trait ExprExt {
                 let (lp, lv) = left.as_bool();
                 let (rp, rv) = right.as_bool();
 
-                let v = if op == op!("&") {
+                let v = if *op == op!("&") {
                     lv.and(rv)
                 } else {
                     lv.or(rv)
@@ -541,6 +607,48 @@ pub trait ExprExt {
                 Unknown
             }
 
+            Expr::Bin(BinExpr {
+                ref left,
+                op: op!("&&"),
+                ref right,
+                ..
+            }) => {
+                let (lp, lv) = left.as_bool();
+                if let Known(false) = lv {
+                    return (lp, lv);
+                }
+
+                let (rp, rv) = right.as_bool();
+                if let Known(false) = rv {
+                    return (lp + rp, rv);
+                }
+
+                Unknown
+            }
+
+            Expr::Bin(BinExpr {
+                left,
+                op: op!(bin, "+"),
+                right,
+                ..
+            }) => {
+                match &**left {
+                    Expr::Lit(Lit::Str(s)) if !s.value.is_empty() => {
+                        return (MayBeImpure, Known(true))
+                    }
+                    _ => {}
+                }
+
+                match &**right {
+                    Expr::Lit(Lit::Str(s)) if !s.value.is_empty() => {
+                        return (MayBeImpure, Known(true))
+                    }
+                    _ => {}
+                }
+
+                Unknown
+            }
+
             Expr::Fn(..) | Expr::Class(..) | Expr::New(..) | Expr::Array(..) | Expr::Object(..) => {
                 Known(true)
             }
@@ -581,25 +689,24 @@ pub trait ExprExt {
         }
     }
 
-    /// Emulates javascript Number() cast function.
-    fn as_number(&self) -> Value<f64> {
+    fn cast_to_number(&self) -> (Purity, Value<f64>) {
         let expr = self.as_expr();
-        let v = match *expr {
-            Expr::Lit(ref l) => match *l {
+        let v = match expr {
+            Expr::Lit(l) => match l {
                 Lit::Bool(Bool { value: true, .. }) => 1.0,
                 Lit::Bool(Bool { value: false, .. }) | Lit::Null(..) => 0.0,
-                Lit::Num(Number { value: n, .. }) => n,
-                Lit::Str(Str { ref value, .. }) => return num_from_str(value),
-                _ => return Unknown,
+                Lit::Num(Number { value: n, .. }) => *n,
+                Lit::Str(Str { value, .. }) => return (Pure, num_from_str(&value)),
+                _ => return (Pure, Unknown),
             },
-            Expr::Ident(Ident { ref sym, .. }) => match *sym {
+            Expr::Ident(Ident { sym, .. }) => match *sym {
                 js_word!("undefined") | js_word!("NaN") => NAN,
                 js_word!("Infinity") => INFINITY,
-                _ => return Unknown,
+                _ => return (Pure, Unknown),
             },
             Expr::Unary(UnaryExpr {
                 op: op!(unary, "-"),
-                ref arg,
+                arg,
                 ..
             }) if match &**arg {
                 Expr::Ident(Ident {
@@ -623,7 +730,7 @@ pub trait ExprExt {
                         1.0
                     }
                 }
-                _ => return Unknown,
+                _ => return (MayBeImpure, Unknown),
             },
             Expr::Unary(UnaryExpr {
                 op: op!("void"),
@@ -631,23 +738,49 @@ pub trait ExprExt {
                 ..
             }) => {
                 if arg.may_have_side_effects() {
-                    return Unknown;
+                    return (MayBeImpure, Known(NAN));
                 } else {
                     NAN
                 }
             }
 
             Expr::Tpl(..) | Expr::Object(ObjectLit { .. }) | Expr::Array(ArrayLit { .. }) => {
-                return num_from_str(&*match self.as_string() {
-                    Known(v) => v,
-                    Unknown => return Value::Unknown,
-                });
+                return (
+                    Pure,
+                    num_from_str(&*match self.as_string() {
+                        Known(v) => v,
+                        Unknown => return (MayBeImpure, Unknown),
+                    }),
+                );
             }
 
-            _ => return Unknown,
+            Expr::Seq(seq) => {
+                if let Some(last) = seq.exprs.last() {
+                    let (_, v) = last.cast_to_number();
+
+                    // TODO: Purity
+                    return (MayBeImpure, v);
+                }
+
+                return (MayBeImpure, Unknown);
+            }
+
+            _ => return (MayBeImpure, Unknown),
         };
 
-        Known(v)
+        (Purity::Pure, Known(v))
+    }
+
+    /// Emulates javascript Number() cast function.
+    ///
+    /// Note: This method returns [Known] only if it's pure.
+    fn as_number(&self) -> Value<f64> {
+        let (purity, v) = self.cast_to_number();
+        if !purity.is_pure() {
+            return Unknown;
+        }
+
+        v
     }
 
     /// Returns Known only if it's pure.
@@ -1012,7 +1145,7 @@ pub trait ExprExt {
             }
             Expr::OptChain(ref e) => e.expr.may_have_side_effects(),
 
-            Expr::Invalid(..) => unreachable!(),
+            Expr::Invalid(..) => true,
         }
     }
 }
